@@ -1,20 +1,23 @@
-/*
- *  Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
+/* This file is part of Spectacle, the KDE screenshot utility
+ * Copyright 2019 David Redondo <kde@david-redondo.de>
+ * Copyright (C) 2015 Boudhayan Gupta <bgupta@kde.org>
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU Lesser General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor,
- *  Boston, MA 02110-1301, USA.
+ * You should have received a copy of the GNU Lesser General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
+ *
+ * SPDX-License-Identifier: LGPL-2.0-or-later
  */
 
 #include "KSMainWindow.h"
@@ -22,36 +25,38 @@
 #include "Config.h"
 #include "SettingsDialog/SettingsDialog.h"
 
-#include <KAboutData>
-#include <KGuiItem>
-#include <KHelpMenu>
-#include <KIO/OpenFileManagerWindowJob>
-#include <KLocalizedString>
-#include <KNS3/KMoreToolsMenuFactory>
-#include <KStandardAction>
-#include <KStandardGuiItem>
-#include <KWindowSystem>
-
+#include <QApplication>
 #include <QClipboard>
 #include <QDesktopServices>
-#include <QJsonArray>
+#include <QDBusConnection>
+#include <QDBusMessage>
 #include <QPrintDialog>
 #include <QPushButton>
 #include <QTimer>
+#include <QtMath>
+#include <QVariantAnimation>
 #include <QVBoxLayout>
+
 #ifdef XCB_FOUND
 #include <QX11Info>
 #include <xcb/xcb.h>
 #endif
 
+#include <KAboutData>
+#include <KGuiItem>
+#include <KHelpMenu>
+#include <KIO/OpenFileManagerWindowJob>
+#include <KLocalizedString>
+#include <KStandardAction>
+#include <KWindowSystem>
 
 static const int DEFAULT_WINDOW_HEIGHT = 420;
 static const int DEFAULT_WINDOW_WIDTH = 840;
 static const int MAXIMUM_WINDOW_WIDTH = 1000;
 
-KSMainWindow::KSMainWindow(const QVector<ImageGrabber::GrabMode>& supportedModes, bool onClickAvailable, QWidget *parent) :
+KSMainWindow::KSMainWindow(const Platform::GrabModes &theGrabModes, const Platform::ShutterModes &theShutterModes, QWidget *parent) :
     QDialog(parent),
-    mKSWidget(new KSWidget(supportedModes, this)),
+    mKSWidget(new KSWidget(theGrabModes, this)),
     mDivider(new QFrame(this)),
     mDialogButtonBox(new QDialogButtonBox(this)),
     mConfigureButton(new QToolButton(this)),
@@ -66,7 +71,7 @@ KSMainWindow::KSMainWindow(const QVector<ImageGrabber::GrabMode>& supportedModes
     mToolsMenu(new QMenu(this)),
     mScreenRecorderToolsMenu(new QMenu(this)),
     mExportMenu(new ExportMenu(this)),
-    mOnClickAvailable(onClickAvailable)
+    mShutterModes(theShutterModes)
 {
     // before we do anything, we need to set a window property
     // that skips the close/hide window animation on kwin. this
@@ -86,7 +91,6 @@ KSMainWindow::KSMainWindow(const QVector<ImageGrabber::GrabMode>& supportedModes
         }
 
         // do the xcb shenanigans
-
         xcb_connection_t *xcbConn = QX11Info::connection();
         const QByteArray effectName = QByteArrayLiteral("_KDE_NET_WM_SKIP_CLOSE_ANIMATION");
 
@@ -105,9 +109,6 @@ KSMainWindow::KSMainWindow(const QVector<ImageGrabber::GrabMode>& supportedModes
     QMetaObject::invokeMethod(this, "init", Qt::QueuedConnection);
 }
 
-KSMainWindow::~KSMainWindow()
-{}
-
 // GUI init
 
 void KSMainWindow::init()
@@ -123,7 +124,7 @@ void KSMainWindow::init()
 
     // change window title on save
 
-    connect(ExportManager::instance(), &ExportManager::imageSaved, this, &KSMainWindow::setScreenshotWindowTitle);
+    connect(ExportManager::instance(), &ExportManager::imageSaved, this, &KSMainWindow::imageSaved);
 
     // the KSGWidget
 
@@ -171,7 +172,8 @@ void KSMainWindow::init()
                           this, &KSMainWindow::openScreenshotsFolder);
     mToolsMenu->addAction(KStandardAction::print(this, &KSMainWindow::showPrintDialog, this));
     mScreenRecorderToolsMenu = mToolsMenu->addMenu(i18n("Record Screen"));
-    connect(mScreenRecorderToolsMenu, &QMenu::aboutToShow, [this]()
+    mScreenRecorderToolsMenu->setIcon(QIcon::fromTheme(QStringLiteral("media-record")));
+    connect(mScreenRecorderToolsMenu, &QMenu::aboutToShow, this, [this]()
     {
         KMoreToolsMenuFactory *moreToolsMenuFactory = new KMoreToolsMenuFactory(QStringLiteral("spectacle/screenrecorder-tools"));
         moreToolsMenuFactory->setParentWidget(this);
@@ -191,7 +193,6 @@ void KSMainWindow::init()
     connect(mMessageWidget, &KMessageWidget::linkActivated, this, [](const QString &str) { QDesktopServices::openUrl(QUrl(str)); } );
 
     // layouts
-
     mDivider->setFrameShape(QFrame::HLine);
     mDivider->setLineWidth(2);
 
@@ -203,14 +204,14 @@ void KSMainWindow::init()
     mMessageWidget->hide();
 
     // populate our send-to actions
-
     mSendToButton->setMenu(mExportMenu);
     connect(mExportMenu, &ExportMenu::imageShared, this, &KSMainWindow::showImageSharedFeedback);
 
-    // disable onClick mode if not available on the platform
-
-    if (!mOnClickAvailable) {
-        mKSWidget->disableOnClick();
+    // lock down the onClick mode depending on available shutter modes
+    if (!mShutterModes.testFlag(Platform::ShutterMode::OnClick)) {
+        mKSWidget->lockOnClickDisabled();
+    } else if (!mShutterModes.testFlag(Platform::ShutterMode::Immediate)) {
+        mKSWidget->lockOnClickEnabled();
     }
     resize(QSize(DEFAULT_WINDOW_WIDTH, DEFAULT_WINDOW_HEIGHT).expandedTo(minimumSize()));
 
@@ -219,6 +220,10 @@ void KSMainWindow::init()
     actionQuit->setShortcut(QKeySequence::Quit);
     addAction(actionQuit);
 
+    mHideMessageWidgetTimer = new QTimer(this);
+    connect(mHideMessageWidgetTimer, &QTimer::timeout,
+            mMessageWidget, &KMessageWidget::animatedHide);
+    mHideMessageWidgetTimer->setInterval(10000);
     // done with the init
 }
 
@@ -256,7 +261,7 @@ void KSMainWindow::setDefaultSaveAction()
 
 void KSMainWindow::moveEvent(QMoveEvent *event)
 {
-    Q_UNUSED(event);
+    Q_UNUSED(event)
 
     KSharedConfigPtr config = KSharedConfig::openConfig(QStringLiteral("spectaclerc"));
     KConfigGroup guiConfig(config, "GuiConfig");
@@ -266,25 +271,75 @@ void KSMainWindow::moveEvent(QMoveEvent *event)
 }
 
 // slots
-
-void KSMainWindow::captureScreenshot(ImageGrabber::GrabMode mode, int timeout, bool includePointer, bool includeDecorations)
+void KSMainWindow::captureScreenshot(Spectacle::CaptureMode theCaptureMode, int theTimeout, bool theIncludePointer, bool theIncludeDecorations)
 {
-    hide();
+    if (theTimeout < 0) { // OnClick is checked (always the case on Wayland)
+        hide();
+        emit newScreenshotRequest(theCaptureMode, theTimeout, theIncludePointer, theIncludeDecorations);
+        return;
+    }
+
+    showMinimized();
     mMessageWidget->hide();
-    emit newScreenshotRequest(mode, timeout, includePointer, includeDecorations);
+    QTimer* timer = new QTimer;
+    timer->setSingleShot(true);
+    timer->setInterval(theTimeout);
+    auto unityUpdate = [](const QVariantMap &properties) {
+        QDBusMessage message = QDBusMessage::createSignal(QStringLiteral("/org/kde/Spectacle"),
+            QStringLiteral("com.canonical.Unity.LauncherEntry"), QStringLiteral("Update"));
+        message.setArguments({QGuiApplication::desktopFileName(), properties});
+        QDBusConnection::sessionBus().send(message);
+    };
+    auto delayAnimation = new QVariantAnimation(timer);
+    delayAnimation->setStartValue(0.0);
+    delayAnimation->setEndValue(1.0);
+    delayAnimation->setDuration(timer->interval());
+    connect(delayAnimation, &QVariantAnimation::valueChanged, this, [=] {
+        const double progress = delayAnimation->currentValue().toDouble();
+        const double timeoutInSeconds = theTimeout / 1000.0;
+        unityUpdate({ {QStringLiteral("progress"), progress} });
+        setWindowTitle(i18ncp("@title:window", "%1 second", "%1 seconds",
+            qMin(int(timeoutInSeconds), qCeil((1 - progress) * timeoutInSeconds))));
+    });
+    connect(timer, &QTimer::timeout, this, [=] {
+        this->hide();
+        timer->deleteLater();
+        unityUpdate({ {QStringLiteral("progress-visible"), false} });
+        emit newScreenshotRequest(theCaptureMode, 0, theIncludePointer, theIncludeDecorations);
+    });
+
+    connect(mKSWidget, &KSWidget::screenshotCanceled, timer, [=] {
+        timer->stop();
+        timer->deleteLater();
+        restoreWindowTitle();
+        unityUpdate({ {QStringLiteral("progress-visible"), false} });
+    });
+
+
+    unityUpdate({   {QStringLiteral("progress-visible"), true},
+                    {QStringLiteral("progress"), 0 } });
+    timer->start();
+    delayAnimation->start();
 }
 
 void KSMainWindow::setScreenshotAndShow(const QPixmap &pixmap)
 {
-    mKSWidget->setScreenshotPixmap(pixmap);
-    mExportMenu->imageUpdated();
-
-    setWindowTitle(i18nc("@title:window Unsaved Screenshot", "Unsaved[*]"));
-    setWindowModified(true);
-
+    if (!pixmap.isNull()) {
+        mKSWidget->setScreenshotPixmap(pixmap);
+        mExportMenu->imageUpdated();
+        setWindowTitle(i18nc("@title:window Unsaved Screenshot", "Unsaved[*]"));
+        setWindowModified(true);
+    } else {
+        restoreWindowTitle();
+    }
+    mKSWidget->setButtonState(KSWidget::State::TakeNewScreenshot);
     show();
-
-    resize(QSize(windowWidth(pixmap), DEFAULT_WINDOW_HEIGHT));
+    activateWindow();
+    /* NOTE windowWidth only produces the right result if it is called after the window is visible.
+     * Because of this the call is not moved into the if above */
+    if(!pixmap.isNull()) {
+        resize(QSize(windowWidth(pixmap), DEFAULT_WINDOW_HEIGHT));
+    }
 }
 
 void KSMainWindow::showPrintDialog()
@@ -313,7 +368,7 @@ void KSMainWindow::openScreenshotsFolder()
     case SaveMode::Save:
         location = cfgManager->lastSaveFile();
         if (!exportManager->isFileExists(location)) {
-            location = QUrl(cfgManager->defaultSaveLocation());
+            location = cfgManager->defaultSaveLocation();
         }
         break;
     case SaveMode::SaveAs:
@@ -343,8 +398,16 @@ void KSMainWindow::quit(const QuitBehavior quitBehavior)
     // is not working quite right, see Bug #389694 which needs fixing.
 }
 
-void KSMainWindow::showInlineMessage(const QString& message, const KMessageWidget::MessageType messageType, const MessageDuration messageDuration)
+void KSMainWindow::showInlineMessage(const QString& message, const KMessageWidget::MessageType messageType,
+                                     const MessageDuration messageDuration, const QList<QAction*>& actions)
 {
+    const auto messageWidgetActions = mMessageWidget->actions();
+    for (QAction* action: messageWidgetActions) {
+        mMessageWidget->removeAction(action);
+    }
+    for (QAction* action : actions) {
+        mMessageWidget->addAction(action);
+    }
     mMessageWidget->setText(message);
     mMessageWidget->setMessageType(messageType);
 
@@ -361,18 +424,22 @@ void KSMainWindow::showInlineMessage(const QString& message, const KMessageWidge
     case KMessageWidget::Information:
         mMessageWidget->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
         break;
-    default:
-        ;
     }
 
+    mHideMessageWidgetTimer->stop();
     mMessageWidget->animatedShow();
     if (messageDuration == MessageDuration::AutoHide) {
-        QTimer::singleShot(10000, mMessageWidget, &KMessageWidget::animatedHide);
+        mHideMessageWidgetTimer->start();
     }
 }
 
 void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
 {
+    if (error == 1) {
+        // error == 1 means the user cancelled the sharing
+        return;
+    }
+
     if (error) {
         showInlineMessage(i18n("There was a problem sharing the image: %1", message),
                           KMessageWidget::Error);
@@ -389,7 +456,8 @@ void KSMainWindow::showImageSharedFeedback(bool error, const QString &message)
 
 void KSMainWindow::sendToClipboard()
 {
-    ExportManager::instance()->doCopyToClipboard();
+    bool notify = false;
+    ExportManager::instance()->doCopyToClipboard(notify);
 
     SpectacleConfig::instance()->quitAfterSaveOrCopyChecked()
             ? quit()
@@ -403,10 +471,15 @@ void KSMainWindow::showPreferencesDialog()
     prefDialog.exec();
 }
 
-void KSMainWindow::setScreenshotWindowTitle(const QUrl &location)
+void KSMainWindow::imageSaved(const QUrl &location)
 {
     setWindowTitle(location.fileName());
     setWindowModified(false);
+    QAction* openContaining = new QAction(QIcon::fromTheme(QStringLiteral("document-open-folder")), i18n("Open Containing Folder"), mMessageWidget);
+    connect(openContaining, &QAction::triggered, [=] { KIO::highlightInFileManager({location});});
+    showInlineMessage(i18n("The screenshot was saved as <a href=\"%1\">%2</a>",
+                           location.toString(), location.fileName()), KMessageWidget::Positive,
+                           MessageDuration::AutoHide, {openContaining});
 }
 
 void KSMainWindow::save()
@@ -431,3 +504,12 @@ void KSMainWindow::saveAs()
         quit(QuitBehavior::QuitExternally);
     }
 }
+
+void KSMainWindow::restoreWindowTitle()
+{
+    if (isWindowModified()) {
+        setWindowTitle(i18nc("@title:window Unsaved Screenshot", "Unsaved[*]"));
+    } else {
+        setWindowTitle(SpectacleConfig::instance()->lastSaveFile().fileName());
+    }
+}
