@@ -226,7 +226,7 @@ void SpectacleCore::screenshotUpdated(const QPixmap &thePixmap)
             mQuickEditor = std::make_unique<QuickEditor>(thePixmap);
             connect(mQuickEditor.get(), &QuickEditor::grabDone, this, &SpectacleCore::screenshotUpdated);
             connect(mQuickEditor.get(), &QuickEditor::grabCancelled, this, &SpectacleCore::screenshotFailed);
-            mQuickEditor->showFullScreen();
+            mQuickEditor->show();
             return;
         } else {
             mQuickEditor->hide();
@@ -327,12 +327,21 @@ void SpectacleCore::doNotify(const QUrl &theSavedAt)
         connect(lNotify, QOverload<uint>::of(&KNotification::activated), this, [this, theSavedAt](uint index) {
             if (index == 0) {
                 new KRun(theSavedAt, nullptr);
-                QTimer::singleShot(250, this, &SpectacleCore::allDone);
+                QTimer::singleShot(250, this, [this] {
+                    if (mStartMode != StartMode::Gui) {
+                        emit allDone();
+                    }
+                });
             }
         });
     }
 
-    connect(lNotify, &QObject::destroyed, this, &SpectacleCore::allDone);
+    connect(lNotify, &QObject::destroyed, this, [this] {
+        if (mStartMode != StartMode::Gui) {
+            emit allDone();
+        }
+    });
+
     lNotify->sendEvent();
 }
 
